/*******************************************************************************
*
*	TITLE:		ir_sensor.h
*
*	VERSION:	0.1 (Beta)                           
*
*	DATE:		22-Feb-2008
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This software provides an interface to a Vishay infrared 
*				sensor which can be used to decode commands sent from a 
*				Sony television remote control or one on the many universal
*				remote controls configured to communicate with a Sony 
*				device. It receives Vishay TSOP4840 (or similar) infrared 
*				sensor waveforms encoded with the very common Sony 12-bit
*				remote control protocol and outputs the 5-bit address and 
*				7-bit command.
*
*				This software uses timer 1.
*
*				This software assumes that your Vishay IR sensor is
*				attached to digital I/O pins 3, 4, 5 or 6. See included
*				Vishay connection diagram document for instructions.
*
*				This file best viewed with tabs set to four.
*
*				You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or elsewhere
*				without permission. Thanks.
*
*				Copyright 2008 R. Kevin Watson. All rights are reserved.
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	22-Feb-2008  0.1  RKW - Original code.
*
*******************************************************************************/
#ifndef _ir_sensor_h
#define _ir_sensor_h

// This is where the three different valid pulse widths
// are defined. The units are in timer "ticks", each of
// which is equal to 100ns. It should be noted that the
// pulse we measure starts on the negative going (falling)
// edge and ends at the positive going (rising) edge.

// #define USE_CUSTOM_TOLERANCE
// #define USE_5_PERCENT_TOLERANCE
// #define USE_10_PERCENT_TOLERANCE
// #define USE_15_PERCENT_TOLERANCE
   #define USE_20_PERCENT_TOLERANCE

#ifdef USE_CUSTOM_TOLERANCE
#define LOGIC_ZERO_TICKS_LOW	 6365	// 0.67ms-5% = 0.6365ms
#define LOGIC_ZERO_TICKS_HIGH	 7035	// 0.67ms+5% = 0.7035ms
#define LOGIC_ONE_TICKS_LOW		11970	// 1.26ms-5% = 1.1970ms
#define LOGIC_ONE_TICKS_HIGH	13230	// 1.26ms+5% = 1.3230ms
#define START_BIT_TICKS_LOW		23370	// 2.46ms-5% = 2.3370ms
#define START_BIT_TICKS_HIGH	27300	// 2.46ms+5% = 2.7300ms
#endif

#ifdef USE_5_PERCENT_TOLERANCE
#define LOGIC_ZERO_TICKS_LOW	 5700	// 0.6ms-5% = 0.5700ms
#define LOGIC_ZERO_TICKS_HIGH	 6300	// 0.6ms+5% = 0.6300ms
#define LOGIC_ONE_TICKS_LOW		11400	// 1.2ms-5% = 1.1400ms
#define LOGIC_ONE_TICKS_HIGH	12600	// 1.2ms+5% = 1.2600ms
#define START_BIT_TICKS_LOW		22800	// 2.4ms-5% = 2.2800ms
#define START_BIT_TICKS_HIGH	25200	// 2.4ms+5% = 2.5200ms
#endif

#ifdef USE_10_PERCENT_TOLERANCE
#define LOGIC_ZERO_TICKS_LOW	 5400	// 0.6ms-10% = 0.5400ms
#define LOGIC_ZERO_TICKS_HIGH	 6600	// 0.6ms+10% = 0.6600ms
#define LOGIC_ONE_TICKS_LOW		10800	// 1.2ms-10% = 1.0800ms
#define LOGIC_ONE_TICKS_HIGH	13200	// 1.2ms+10% = 1.3200ms
#define START_BIT_TICKS_LOW		21600	// 2.4ms-10% = 2.1600ms
#define START_BIT_TICKS_HIGH	26400	// 2.4ms+10% = 2.6400ms
#endif

#ifdef USE_15_PERCENT_TOLERANCE
#define LOGIC_ZERO_TICKS_LOW	 5100	// 0.6ms-15% = 0.5100ms
#define LOGIC_ZERO_TICKS_HIGH	 6900	// 0.6ms+15% = 0.6900ms
#define LOGIC_ONE_TICKS_LOW		10200	// 1.2ms-15% = 1.0200ms
#define LOGIC_ONE_TICKS_HIGH	13800	// 1.2ms+15% = 1.3800ms
#define START_BIT_TICKS_LOW		20400	// 2.4ms-15% = 2.0400ms
#define START_BIT_TICKS_HIGH	27600	// 2.4ms+15% = 2.7600ms
#endif

#ifdef USE_20_PERCENT_TOLERANCE
#define LOGIC_ZERO_TICKS_LOW	 4800	// 0.6ms-20% = 0.4800ms
#define LOGIC_ZERO_TICKS_HIGH	 7200	// 0.6ms+20% = 0.7200ms
#define LOGIC_ONE_TICKS_LOW		 9600	// 1.2ms-20% = 0.9600ms
#define LOGIC_ONE_TICKS_HIGH	14400	// 1.2ms+20% = 1.4400ms
#define START_BIT_TICKS_LOW		19200	// 2.4ms-20% = 1.9200ms
#define START_BIT_TICKS_HIGH	28800	// 2.4ms+20% = 2.8800ms
#endif


// structures
typedef struct
{
	unsigned char command;
	unsigned char address;
} Sony_IR_Data_Struct;

typedef Sony_IR_Data_Struct *Sony_IR_Data_Ptr;

extern Sony_IR_Data_Struct ir_data;

// function prototypes
void IR_Sensor_ISR(unsigned char);
void Initialize_IR_Sensor(void);
void Get_IR_Sensor_Data(Sony_IR_Data_Ptr);
unsigned char Get_IR_Sensor_Count(void);
void Reset_IR_Sensor_Count(void);

#endif
